@@ -147,6 +147,10 @@
 /*	int	warn_compat_break_flush_domains;
 /*	int	warn_compat_break_mynetworks_style;
 /*
+/*	int	warn_compat_break_smtpd_tls_fpt_dgst;
+/*	int	warn_compat_break_smtp_tls_fpt_dgst;
+/*	int	warn_compat_break_lmtp_tls_fpt_dgst;
+/*
 /*	char	*var_maillog_file;
 /*	char	*var_maillog_file_pfxs;
 /*	char	*var_maillog_file_comp;
@@ -364,6 +368,13 @@ char	*var_postlog_service;
 
 const char null_format_string[1] = "";
 
+ /*
+  * Compatibility level 3.
+  */
+int	warn_compat_break_smtpd_tls_fpt_dgst;
+int	warn_compat_break_smtp_tls_fpt_dgst;
+int	warn_compat_break_lmtp_tls_fpt_dgst;
+
  /*
   * Compatibility level 2.
   */
@@ -618,6 +629,23 @@ static void check_legacy_defaults(void)
      * bits.
      */
 
+    /*
+     * Look for specific parameters whose default changed when the
+     * compatibility level changed to 3.
+     */
+    if (var_compat_level < 3) {
+	if (mail_conf_lookup(VAR_SMTPD_TLS_FPT_DGST) == 0)
+	    warn_compat_break_smtpd_tls_fpt_dgst = 1;
+	if (mail_conf_lookup(VAR_SMTP_TLS_FPT_DGST) == 0)
+	    warn_compat_break_smtp_tls_fpt_dgst = 1;
+	if (mail_conf_lookup(VAR_LMTP_TLS_FPT_DGST) == 0)
+	    warn_compat_break_lmtp_tls_fpt_dgst = 1;
+    } else {
+	warn_compat_break_smtpd_tls_fpt_dgst = 0;
+	warn_compat_break_smtp_tls_fpt_dgst = 0;
+	warn_compat_break_lmtp_tls_fpt_dgst = 0;
+    }
+
     /*
      * Look for specific parameters whose default changed when the
      * compatibility level changed to 2.
@@ -871,6 +899,8 @@ void    mail_params_init()
     var_smtputf8_enable = 0;
 #else
     midna_domain_transitional = var_idna2003_compat;
+    if (var_smtputf8_enable)
+	midna_domain_pre_chroot();
 #endif
     util_utf8_enable = var_smtputf8_enable;
 
@@ -974,10 +1004,6 @@ void    mail_params_init()
 	msg_fatal("%s parameter setting must not contain multiple values: %s",
 		  VAR_MYORIGIN, var_myorigin);
 
-    if (var_relayhost[strcspn(var_relayhost, CHARS_COMMA_SP)])
-	msg_fatal("%s parameter setting must not contain multiple values: %s",
-		  VAR_RELAYHOST, var_relayhost);
-
     /*
      * One more sanity check.
      */
