@@ -74,6 +74,11 @@
 /*	IBM T.J. Watson Research
 /*	P.O. Box 704
 /*	Yorktown Heights, NY 10598, USA
+/*
+/*	Wietse Venema
+/*	Google, Inc.
+/*	111 8th Avenue
+/*	New York, NY 10011, USA
 /*--*/
 
 /* System library. */
@@ -103,9 +108,8 @@
 #define PEM_LOAD_STATE_CERT	3	/* Last object was a certificate */
 #define PEM_LOAD_STATE_BOTH	4	/* Unordered, key + first cert seen */
 
-#define PEM_LOAD_MODE_MIXED	0	/* Key order not fixed */
-#define PEM_LOAD_MODE_MORE	1	/* Keys first, more files */
-#define PEM_LOAD_MODE_LAST	2	/* Keys first, last file */
+#define PEM_LOAD_READ_LAST	0	/* Reading last file */
+#define PEM_LOAD_READ_MORE	1	/* More files to be read */
 
 typedef struct pem_load_state_t {
     const char *origin;			/* PEM chain origin description */
@@ -120,6 +124,7 @@ typedef struct pem_load_state_t {
     int     keynum;			/* Index of last key */
     int     objnum;			/* Index in current source */
     int     state;			/* Current state, never "DONE" */
+    int     mixed;			/* Single file with key anywhere */
 } pem_load_state_t;
 
 /* init_pem_load_state - fill in initial pem_load_state structure */
@@ -139,11 +144,11 @@ static void init_pem_load_state(pem_load_state_t *st, SSL_CTX *ctx, SSL *ssl,
     st->keynum = 0;
     st->objnum = 0;
     st->state = PEM_LOAD_STATE_INIT;
+    st->mixed = 0;
 }
 
 /* use_chain - load cert, key and chain into ctx or ssl */
 
-#if OPENSSL_VERSION_NUMBER >= 0x1010100fUL
 static int use_chain(pem_load_state_t *st)
 {
     int     ret;
@@ -175,53 +180,9 @@ static int use_chain(pem_load_state_t *st)
     return ret;
 }
 
-#else
-
-/* Legacy OpenSSL 1.0.2 and 1.1.0 interface */
-static int use_chain(pem_load_state_t *st)
-{
-    int     ret = 1;
-
-#define TRY(op, o) \
-    ((st->ctx && SSL_CTX_##op(st->ctx, st->o)) || \
-     (st->ssl && SSL_##op(st->ssl, st->o)))
-
-    /*
-     * This ensures the cert and key have the same type and match. A similar
-     * check is performed in use_PrivateKey(), but only if if the key and
-     * cert are of the same type.
-     */
-    if (!X509_check_private_key(st->cert, st->pkey))
-	ret = 0;
-
-    /*
-     * XXX: With OpenSSL 1.0.2, setting the certificate clears any previous
-     * mismatched key of the same type, so we don't detect conflicting chains
-     * for the same algorithm, and silently use the last one.
-     */
-
-    /* use_certificate() increments the refcount */
-    if (ret && !TRY(use_certificate, cert))
-	ret = 0;
-    X509_free(st->cert);
-
-    /* use_PrivateKey() increments the refcount */
-    if (ret && !TRY(use_PrivateKey, pkey))
-	ret = 0;
-    EVP_PKEY_free(st->pkey);
-
-    /* set0_chain() does not increment the refcount */
-    if (!ret || !(ret = TRY(set0_chain, chain)))
-	sk_X509_pop_free(st->chain, X509_free);
-
-    return ret;
-}
-
-#endif
-
 /* load_cert - decode and load a DER-encoded X509 certificate */
 
-static void load_cert(pem_load_state_t *st, int mode, unsigned char *buf,
+static void load_cert(pem_load_state_t *st, unsigned char *buf,
 		              long buflen)
 {
     const unsigned char *p = buf;
@@ -229,9 +190,9 @@ static void load_cert(pem_load_state_t *st, int mode, unsigned char *buf,
 
     /*
      * When expecting one or more keys, each key must precede the associated
-     * certicate (chain).
+     * certificate (chain).
      */
-    if (mode != PEM_LOAD_MODE_MIXED && st->state == PEM_LOAD_STATE_INIT) {
+    if (!st->mixed && st->state == PEM_LOAD_STATE_INIT) {
 	msg_warn("error loading chain from %s: key not first", st->source);
 	if (cert)
 	    X509_free(cert);
@@ -254,36 +215,34 @@ static void load_cert(pem_load_state_t *st, int mode, unsigned char *buf,
 
     /*
      * The first certificate after a new key becomes the leaf certificate for
-     * that key, and the additional issuer certificate list is cleared.
-     * Subsequent certificates are added to the issuer chain (without an
-     * "upref" so they are now owne by the chain, and must not be freed).
+     * that key.  Subsequent certificates are added to the issuer chain.
      * 
      * In "mixed" mode, the first certificate is either after the key, or else
      * comes first.
      */
-    if (!st->cert) {
+    switch (st->state) {
+    case PEM_LOAD_STATE_PKEY:
 	st->cert = cert;
-    } else if ((!st->chain && (st->chain = sk_X509_new_null()) == 0)
-	       || !sk_X509_push(st->chain, cert)) {
-	X509_free(cert);
-	st->state = PEM_LOAD_STATE_FAIL;
+	st->state = st->mixed ? PEM_LOAD_STATE_BOTH : PEM_LOAD_STATE_CERT;
 	return;
-    }
-    if (st->state == PEM_LOAD_STATE_INIT) {
+    case PEM_LOAD_STATE_INIT:
+	st->cert = cert;
 	st->state = PEM_LOAD_STATE_CERT;
 	return;
-    }
-    if (st->state != PEM_LOAD_STATE_PKEY)
+    case PEM_LOAD_STATE_CERT:
+    case PEM_LOAD_STATE_BOTH:
+	if ((!st->chain && (st->chain = sk_X509_new_null()) == 0)
+	    || !sk_X509_push(st->chain, cert)) {
+	    X509_free(cert);
+	    st->state = PEM_LOAD_STATE_FAIL;
+	}
 	return;
-    if (mode == PEM_LOAD_MODE_MIXED)
-	st->state = PEM_LOAD_STATE_BOTH;
-    else
-	st->state = PEM_LOAD_STATE_CERT;
+    }
 }
 
 /* load_pkey - decode and load a DER-encoded private key */
 
-static void load_pkey(pem_load_state_t *st, int mode, int pkey_type,
+static void load_pkey(pem_load_state_t *st, int pkey_type,
 		              unsigned char *buf, long buflen)
 {
     const char *myname = "load_pkey";
@@ -315,23 +274,45 @@ static void load_pkey(pem_load_state_t *st, int mode, int pkey_type,
      * ignored with a warning.
      */
     switch (st->state) {
-    case PEM_LOAD_STATE_INIT:
-	break;
-
     case PEM_LOAD_STATE_CERT:
 
 	/*
 	 * When processing the key of a "next" chain, we're in the "CERT"
 	 * state, and first complete the processing of the previous chain.
 	 */
-	if (mode != PEM_LOAD_MODE_MIXED && !use_chain(st)) {
+	if (!st->mixed && !use_chain(st)) {
 	    msg_warn("error loading certificate chain: "
-		     "key number %d in %s does not match the certificate",
+		     "key at index %d in %s does not match the certificate",
 		     st->keynum, st->keysrc);
 	    st->state = PEM_LOAD_STATE_FAIL;
 	    return;
 	}
-	break;
+	/* FALLTHROUGH */
+    case PEM_LOAD_STATE_INIT:
+
+	if (!pkey) {
+	    msg_warn("error loading private key (PEM object number %d) from %s",
+		     st->objnum, st->source);
+	    st->state = PEM_LOAD_STATE_FAIL;
+	    return;
+	}
+	/* Reject unexpected data beyond the end of the DER-encoded object */
+	if (p - buf != buflen) {
+	    msg_warn("error loading private key (PEM object number %d) from"
+		     " %s: excess data", st->objnum, st->source);
+	    EVP_PKEY_free(pkey);
+	    st->state = PEM_LOAD_STATE_NOGO;
+	    return;
+	}
+	/* All's well, update the state */
+	st->pkey = pkey;
+	if (st->state == PEM_LOAD_STATE_INIT)
+	    st->state = PEM_LOAD_STATE_PKEY;
+	else if (st->mixed)
+	    st->state = PEM_LOAD_STATE_BOTH;
+	else
+	    st->state = PEM_LOAD_STATE_PKEY;
+	return;
 
     case PEM_LOAD_STATE_PKEY:
     case PEM_LOAD_STATE_BOTH:
@@ -339,14 +320,14 @@ static void load_pkey(pem_load_state_t *st, int mode, int pkey_type,
 	    EVP_PKEY_free(pkey);
 
 	/* XXX: Legacy behaviour was silent, should we stay silent? */
-	if (mode == PEM_LOAD_MODE_MIXED) {
+	if (st->mixed) {
 	    msg_warn("ignoring 2nd key at index %d in %s after 1st at %d",
 		     st->objnum, st->source, st->keynum);
 	    return;
 	}
-	/* Mode != "mixed" => state == "PKEY", so back-to-back keys */
+	/* else back-to-back keys */
 	msg_warn("error loading certificate chain: "
-		 "key number %d in %s not followed by a certificate",
+		 "key at index %d in %s not followed by a certificate",
 		 st->keynum, st->keysrc);
 	st->state = PEM_LOAD_STATE_NOGO;
 	return;
@@ -356,38 +337,11 @@ static void load_pkey(pem_load_state_t *st, int mode, int pkey_type,
 	st->state = PEM_LOAD_STATE_NOGO;
 	return;
     }
-
-    if (!pkey) {
-	msg_warn("error loading private key (PEM object number %d) from %s",
-		 st->objnum, st->source);
-	st->state = PEM_LOAD_STATE_FAIL;
-	return;
-    }
-    /* Reject unexpected data beyond the end of the DER-encoded object */
-    if (p - buf != buflen) {
-	msg_warn("error loading private key (PEM object number %d) from %s:"
-		 " excess data", st->objnum, st->source);
-	EVP_PKEY_free(pkey);
-	st->state = PEM_LOAD_STATE_NOGO;
-	return;
-    }
-    /* All's well, update the state */
-    st->pkey = pkey;
-    if (st->state == PEM_LOAD_STATE_INIT) {
-	st->state = PEM_LOAD_STATE_PKEY;
-	return;
-    }
-    if (st->state != PEM_LOAD_STATE_CERT)
-	return;
-    if (mode == PEM_LOAD_MODE_MIXED)
-	st->state = PEM_LOAD_STATE_BOTH;
-    else
-	st->state = PEM_LOAD_STATE_PKEY;
 }
 
 /* load_pem_object - load next pkey or cert from open BIO */
 
-static int load_pem_object(pem_load_state_t *st, int mode)
+static int load_pem_object(pem_load_state_t *st)
 {
     char   *name = 0;
     char   *header = 0;
@@ -405,7 +359,7 @@ static int load_pem_object(pem_load_state_t *st, int mode)
     }
     if (strcmp(name, PEM_STRING_X509) == 0
 	|| strcmp(name, PEM_STRING_X509_OLD) == 0) {
-	load_cert(st, mode, buf, buflen);
+	load_cert(st, buf, buflen);
     } else if (strcmp(name, PEM_STRING_PKCS8INF) == 0
 	       || ((pkey_type = EVP_PKEY_RSA) != NID_undef
 		   && strcmp(name, PEM_STRING_RSA) == 0)
@@ -413,10 +367,9 @@ static int load_pem_object(pem_load_state_t *st, int mode)
 		   && strcmp(name, PEM_STRING_ECPRIVATEKEY) == 0)
 	       || ((pkey_type = EVP_PKEY_DSA) != NID_undef
 		   && strcmp(name, PEM_STRING_DSA) == 0)) {
-	load_pkey(st, mode, pkey_type, buf, buflen);
-    } else if (mode != PEM_LOAD_MODE_MIXED) {
-	msg_warn("error loading %s: unexpected PEM type: %s", st->source, name);
-	st->state = PEM_LOAD_STATE_NOGO;
+	load_pkey(st, pkey_type, buf, buflen);
+    } else if (!st->mixed) {
+       msg_warn("loading %s: ignoring PEM type: %s", st->source, name);
     }
     OPENSSL_free(name);
     OPENSSL_free(header);
@@ -426,7 +379,7 @@ static int load_pem_object(pem_load_state_t *st, int mode)
 
 /* load_pem_bio - load all key/certs from bio and free the bio */
 
-static int load_pem_bio(pem_load_state_t *st, int mode)
+static int load_pem_bio(pem_load_state_t *st, int more)
 {
     int     state = st->state;
 
@@ -434,71 +387,72 @@ static int load_pem_bio(pem_load_state_t *st, int mode)
     ERR_clear_error();
 
     /*
-     * When "mode" ==PEM_LOAD_MODE_MORE, more files will be loaded after the
+     * When "more" is PEM_LOAD_READ_MORE, more files will be loaded after the
      * current file, and final processing for the last key and chain is
      * deferred.
      * 
-     * When "mode" ==PEM_LOAD_MODE_LAST, this is the last file in the list, and
+     * When "more" is PEM_LOAD_READ_LAST, this is the last file in the list, and
      * we validate the final chain.
      * 
-     * When "mode" ==PEM_LOAD_MODE_MIXED, this is the only file, and its key can
-     * occur at any location.  In this case we load at most one key.
+     * When st->mixed is true, this is the only file, and its key can occur at
+     * any location.  In this case we load at most one key.
      */
     for (st->objnum = 1; state > PEM_LOAD_STATE_DONE; ++st->objnum) {
-	state = load_pem_object(st, mode);
-	if (state == PEM_LOAD_STATE_PKEY || state == PEM_LOAD_STATE_BOTH) {
-	    if (mode != PEM_LOAD_MODE_MIXED
-		|| (mode == PEM_LOAD_MODE_MIXED && st->keynum == 0)) {
-		st->keynum = st->objnum;
-		st->keysrc = st->source;
-	    }
+	state = load_pem_object(st);
+	if ((st->mixed && st->keynum == 0 &&
+	     (state == PEM_LOAD_STATE_PKEY || state == PEM_LOAD_STATE_BOTH))
+	    || (!st->mixed && state == PEM_LOAD_STATE_PKEY)) {
+	    /* Squirrel-away the current key location */
+	    st->keynum = st->objnum;
+	    st->keysrc = st->source;
 	}
     }
     /* We're responsible for unconditionally freeing the BIO */
     BIO_free(st->pembio);
 
     /* Success with current file, go back for more? */
-    if (mode == PEM_LOAD_MODE_MORE && state >= PEM_LOAD_STATE_DONE)
+    if (more == PEM_LOAD_READ_MORE && state >= PEM_LOAD_STATE_DONE)
 	return 0;
 
     /*
      * If all is well so far, complete processing for the final chain.
      */
-    if (st->state >= PEM_LOAD_STATE_INIT) {
-	if (st->state == PEM_LOAD_STATE_INIT) {
-	    msg_warn("No PEM data in %s", st->origin);
-	    st->state = PEM_LOAD_STATE_NOGO;
-	} else if (st->state == PEM_LOAD_STATE_PKEY) {
-	    msg_warn("No certs for key number %d in %s",
-		     st->keynum, st->keysrc);
-	    st->state = PEM_LOAD_STATE_NOGO;
-	} else if (mode == PEM_LOAD_MODE_MIXED && st->state == PEM_LOAD_STATE_CERT) {
+    switch (st->state) {
+    case PEM_LOAD_STATE_FAIL:
+	tls_print_errors();
+	break;
+    default:
+	break;
+    case PEM_LOAD_STATE_INIT:
+	msg_warn("No PEM data in %s", st->origin);
+	break;
+    case PEM_LOAD_STATE_PKEY:
+	msg_warn("No certs for key at index %d in %s", st->keynum, st->keysrc);
+	break;
+    case PEM_LOAD_STATE_CERT:
+	if (st->mixed) {
 	    msg_warn("No private key found in %s", st->origin);
-	    st->state = PEM_LOAD_STATE_NOGO;
-	} else if (!use_chain(st)) {
-	    msg_warn("key number %d in %s does not match next certificate",
-		     st->keynum, st->keysrc);
-	    st->state = PEM_LOAD_STATE_FAIL;
-	} else {
-	    /* use_chain() freed the key and certs, and zeroed the pointers */
-	    return (0);
+	    break;
 	}
+	/* FALLTHROUGH */
+    case PEM_LOAD_STATE_BOTH:
+	/* use_chain() frees the key and certs, and zeroes the pointers */
+	if (use_chain(st))
+	    return (0);
+	msg_warn("key at index %d in %s does not match next certificate",
+		 st->keynum, st->keysrc);
+	tls_print_errors();
+	break;
     }
     /* Free any left-over unused keys and certs */
     EVP_PKEY_free(st->pkey);
     X509_free(st->cert);
     sk_X509_pop_free(st->chain, X509_free);
 
-    switch (st->state) {
-    case PEM_LOAD_STATE_FAIL:
-	tls_print_errors();
-	/* FALLTHROUGH */
-    default:
-	msg_warn("error loading private keys and certificates from: %s: %s",
-	       st->ctx ? "disabling TLS support" : "aborting TLS handshake",
-		 st->origin);
-	return (-1);
-    }
+    msg_warn("error loading private keys and certificates from: %s: %s",
+	     st->origin, st->ctx ? "disabling TLS support" :
+	     "aborting TLS handshake");
+    return (-1);
 }
 
 /* load_chain_files - load sequence of (key, cert, [chain]) from files */
@@ -509,7 +463,7 @@ static int load_chain_files(SSL_CTX *ctx, const char *chain_files)
     ARGV   *files = argv_split(chain_files, CHARS_COMMA_SP);
     char  **filep;
     int     ret = 0;
-    int     mode;
+    int     more;
 
     init_pem_load_state(&st, ctx, 0, chain_files);
     for (filep = files->argv; ret == 0 && *filep; ++filep) {
@@ -519,14 +473,30 @@ static int load_chain_files(SSL_CTX *ctx, const char *chain_files)
 	    st.state = PEM_LOAD_STATE_NOGO;
 	    break;
 	}
-	mode = filep[1] ? PEM_LOAD_MODE_MORE : PEM_LOAD_MODE_LAST;
+	more = filep[1] ? PEM_LOAD_READ_MORE : PEM_LOAD_READ_LAST;
 	/* load_pem_bio() frees the BIO */
-	ret = load_pem_bio(&st, mode);
+	ret = load_pem_bio(&st, more);
     }
     argv_free(files);
     return (ret);
 }
 
+/* load_mixed_file - load certs with single key anywhere in the file */
+
+static int load_mixed_file(SSL_CTX *ctx, const char *file)
+{
+    pem_load_state_t st;
+
+    init_pem_load_state(&st, ctx, 0, file);
+    if ((st.pembio = BIO_new_file(st.source, "r")) == NULL) {
+	msg_warn("error opening chain file: %s: %m", st.source);
+	return (-1);
+    }
+    st.mixed = 1;
+    /* load_pem_bio() frees the BIO */
+    return load_pem_bio(&st, PEM_LOAD_READ_LAST);
+}
+
 /* tls_set_ca_certificate_info - load Certification Authority certificates */
 
 int     tls_set_ca_certificate_info(SSL_CTX *ctx, const char *CAfile,
@@ -573,17 +543,8 @@ static int set_cert_stuff(SSL_CTX *ctx, const char *cert_type,
      * When the certfile and keyfile are one and the same, load both in a
      * single pass, avoiding potential race conditions during key rollover.
      */
-    if (strcmp(cert_file, key_file) == 0) {
-	pem_load_state_t st;
-
-	init_pem_load_state(&st, ctx, 0, cert_file);
-	if ((st.pembio = BIO_new_file(st.source, "r")) == NULL) {
-	    msg_warn("error opening chain file: %s: %m", st.source);
-	    return (-1);
-	}
-	/* load_pem_bio() frees the BIO */
-	return (load_pem_bio(&st, PEM_LOAD_MODE_MIXED) == 0);
-    }
+    if (strcmp(cert_file, key_file) == 0)
+	return (load_mixed_file(ctx, cert_file) == 0);
 
     /*
      * We need both the private key (in key_file) and the public key
@@ -668,7 +629,93 @@ int     tls_load_pem_chain(SSL *ssl, const char *pem, const char *origin)
 	return (-1);
     }
     /* load_pem_bio() frees the BIO */
-    return (load_pem_bio(&st, PEM_LOAD_MODE_LAST));
+    return (load_pem_bio(&st, PEM_LOAD_READ_LAST));
+}
+
+#ifdef TEST
+
+static NORETURN usage(void)
+{
+    fprintf(stderr, "usage: tls_certkey [-m] <chainfiles>\n");
+    exit(1);
+}
+
+int     main(int argc, char *argv[])
+{
+    int     ch;
+    int     mixed = 0;
+    int     ret;
+    char   *key_file = 0;
+    SSL_CTX *ctx;
+
+    if (!(ctx = SSL_CTX_new(TLS_client_method()))) {
+	tls_print_errors();
+	exit(1);
+    }
+    while ((ch = GETOPT(argc, argv, "mk:")) > 0) {
+	switch (ch) {
+	case 'k':
+	    key_file = optarg;
+	    break;
+	case 'm':
+	    mixed = 1;
+	    break;
+	default:
+	    usage();
+	}
+    }
+    argc -= optind;
+    argv += optind;
+
+    if (argc < 1)
+	usage();
+
+    if (key_file)
+	ret = set_cert_stuff(ctx, "any", argv[0], key_file) == 0;
+    else if (mixed)
+	ret = load_mixed_file(ctx, argv[0]);
+    else
+	ret = load_chain_files(ctx, argv[0]);
+
+    if (ret != 0)
+	exit(1);
+
+    if (SSL_CTX_set_current_cert(ctx, SSL_CERT_SET_FIRST) != 1) {
+	fprintf(stderr, "error selecting first certificate\n");
+	tls_print_errors();
+	exit(1);
+    }
+    do {
+	STACK_OF(X509) *chain;
+	int     i;
+
+	if (SSL_CTX_get0_chain_certs(ctx, &chain) != 1) {
+	    fprintf(stderr, "error locating certificate chain\n");
+	    tls_print_errors();
+	    exit(1);
+	}
+	for (i = 0; i <= sk_X509_num(chain); ++i) {
+	    char    buf[CCERT_BUFSIZ];
+	    X509   *cert;
+
+	    if (i > 0)
+		cert = sk_X509_value(chain, i - 1);
+	    else
+		cert = SSL_CTX_get0_certificate(ctx);
+
+	    printf("depth = %d\n", i);
+
+	    X509_NAME_oneline(X509_get_issuer_name(cert), buf, sizeof(buf));
+	    printf("issuer = %s\n", buf);
+
+	    X509_NAME_oneline(X509_get_subject_name(cert), buf, sizeof(buf));
+	    printf("subject = %s\n\n", buf);
+	}
+    } while (SSL_CTX_set_current_cert(ctx, SSL_CERT_SET_NEXT) != 0);
+
+    exit(0);
 }
 
 #endif
+
+#endif
