@@ -333,17 +333,6 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
      */
     tls_check_version();
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-
-    /*
-     * Initialize the OpenSSL library by the book! To start with, we must
-     * initialize the algorithms. We want cleartext error messages instead of
-     * just error codes, so we load the error_strings.
-     */
-    SSL_load_error_strings();
-    OpenSSL_add_ssl_algorithms();
-#endif
-
     /*
      * Create an application data index for SSL objects, so that we can
      * attach TLScontext information; this information is needed inside
@@ -465,19 +454,6 @@ TLS_APPL_STATE *tls_client_init(const TLS_CLIENT_INIT_PROPS *props)
 	return (0);
     }
 
-    /*
-     * 2015-12-05: Ephemeral RSA removed from OpenSSL 1.1.0-dev
-     */
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-
-    /*
-     * According to the OpenSSL documentation, temporary RSA key is needed
-     * export ciphers are in use. We have to provide one, so well, we just do
-     * it.
-     */
-    SSL_CTX_set_tmp_rsa_callback(client_ctx, tls_tmp_rsa_cb);
-#endif
-
     /*
      * With OpenSSL 1.0.2 and later the client EECDH curve list becomes
      * configurable with the preferred curve negotiated via the supported
@@ -907,6 +883,24 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
     if (TLS_DANE_BASED(props->tls_level))
 	protomask |= TLS_PROTOCOL_SSLv2;
 
+    /*
+     * Allocate a new TLScontext for the new connection and get an SSL
+     * structure. Add the location of TLScontext to the SSL to later retrieve
+     * the information inside the tls_verify_certificate_callback().
+     * 
+     * If session caching was enabled when TLS was initialized, the cache type
+     * is stored in the client SSL context.
+     */
+    TLScontext = tls_alloc_sess_context(log_mask, props->namaddr);
+    TLScontext->cache_type = app_ctx->cache_type;
+
+    if ((TLScontext->con = SSL_new(app_ctx->ssl_ctx)) == NULL) {
+	msg_warn("Could not allocate 'TLScontext->con' with SSL_new()");
+	tls_print_errors();
+	tls_free_context(TLScontext);
+	return (0);
+    }
+
     /*
      * Per session cipher selection for sessions with mandatory encryption
      * 
@@ -914,11 +908,11 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
      * to stay the same between connections, so we make use of a 1-element
      * cache to return the same result for identical inputs.
      */
-    cipher_list = tls_set_ciphers(app_ctx, "TLS", props->cipher_grade,
+    cipher_list = tls_set_ciphers(TLScontext, props->cipher_grade,
 				  props->cipher_exclusions);
     if (cipher_list == 0) {
-	msg_warn("%s: %s: aborting TLS session",
-		 props->namaddr, vstring_str(app_ctx->why));
+	/* already warned */
+	tls_free_context(TLScontext);
 	return (0);
     }
     if (log_mask & TLS_LOG_VERBOSE)
@@ -949,17 +943,6 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
      */
     myserverid = tls_serverid_digest(props, protomask, cipher_list);
 
-    /*
-     * Allocate a new TLScontext for the new connection and get an SSL
-     * structure. Add the location of TLScontext to the SSL to later retrieve
-     * the information inside the tls_verify_certificate_callback().
-     * 
-     * If session caching was enabled when TLS was initialized, the cache type
-     * is stored in the client SSL context.
-     */
-    TLScontext = tls_alloc_sess_context(log_mask, props->namaddr);
-    TLScontext->cache_type = app_ctx->cache_type;
-
     TLScontext->serverid = myserverid;
     TLScontext->stream = props->stream;
     TLScontext->mdalg = props->mdalg;
@@ -967,12 +950,6 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
     /* Alias DANE digest info from props */
     TLScontext->dane = props->dane;
 
-    if ((TLScontext->con = SSL_new(app_ctx->ssl_ctx)) == NULL) {
-	msg_warn("Could not allocate 'TLScontext->con' with SSL_new()");
-	tls_print_errors();
-	tls_free_context(TLScontext);
-	return (0);
-    }
     if (!SSL_set_ex_data(TLScontext->con, TLScontext_index, TLScontext)) {
 	msg_warn("Could not set application data for 'TLScontext->con'");
 	tls_print_errors();
@@ -1017,11 +994,19 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
 	 * avoid SNI, and there are no plans to support SNI in the Postfix
 	 * SMTP server).
 	 * 
+	 * Per RFC7672, the required SNI name is the TLSA "base domain" (the one
+	 * used to construct the "_25._tcp.<fqdn>" TLSA record DNS query).
+	 * 
 	 * Since the hostname is DNSSEC-validated, it must be a DNS FQDN and
 	 * thererefore valid for use with SNI.
 	 */
-	sni = props->host;
+	sni = props->dane->base_domain;
     } else if (props->sni && *props->sni) {
+
+	/*
+	 * MTA-STS policy plugin compatibility: with servername=hostname,
+	 * Postfix must send the MX hostname (not CNAME expanded).
+	 */
 	if (strcmp(props->sni, "hostname") == 0)
 	    sni = props->host;
 	else if (strcmp(props->sni, "nexthop") == 0)
@@ -1042,11 +1027,13 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
 	    tls_free_context(TLScontext);
 	    return (0);
 	}
+
 	/*
 	 * The saved value is not presently used client-side, but could later
-	 * be logged if acked by the server (requires new client-side callback
-	 * to detect the ack).  For now this just maintains symmetry with the
-	 * server code, where do record the received SNI for logging.
+	 * be logged if acked by the server (requires new client-side
+	 * callback to detect the ack).  For now this just maintains symmetry
+	 * with the server code, where do record the received SNI for
+	 * logging.
 	 */
 	TLScontext->peer_sni = mystrdup(sni);
 	if (log_mask & TLS_LOG_DEBUG)
@@ -1104,7 +1091,7 @@ TLS_SESS_STATE *tls_client_start(const TLS_CLIENT_START_PROPS *props)
      * Start TLS negotiations. This process is a black box that invokes our
      * call-backs for certificate verification.
      * 
-     * Error handling: If the SSL handhake fails, we print out an error message
+     * Error handling: If the SSL handshake fails, we print out an error message
      * and remove all TLS state concerning this session.
      */
     sts = tls_bio_connect(vstream_fileno(props->stream), props->timeout,
